var path = require('path');
var app = require('app');
var Tray = require('tray');
var Menu = require('menu');
var BrowserWindow = require('browser-window');
var globalShortcut = require('global-shortcut');

var cntserver = require('./cntserver');
var versionInfo = require('./version-info');
var support = require('./support');
var trayUtil = require('./util/tray');
var popupStartLogic = require("./popup").popupStartLogic;
var popupUpdaterCfg = require("./popup").popupUpdaterCfg;
var mainWindow = require('./windows/main');
var deviceWindow = require('./windows/device');
var permitWindow = require('./windows/permit');
var popupWindow = require('./windows/popup');
var translations = require('./translations').translations;

var log = require('../log');
var getAppLang = require("../config/config").getAppLang;
var setAppLang = require("../config/config").setAppLang;
var getAppUUID = require('../config/config').getAppUUID;
var getPermitConfig = require('../config/config').getPermitConfig;
var setPermitConfig = require('../config/config').setPermitConfig;
var getGeneralCfg = require("../config/config").getGeneralCfg;
var setGeneralCfg = require("../config/config").setGeneralCfg;

var imagesDir = path.join(__dirname, "images", process.platform);
var authorized = getGeneralCfg().authorized;
var silentCallbacks = [];
var devices = {};
var notifySocket = null;
var userName = "";
var util = require('util');
var device_id = "this_computer";
var is_link = getPermitConfig().permited;
var appLang = getAppLang();


var show = function () {
    authorized ? mainWindow.show() : popupWindow.show();
};

// Привязка к событиям от cntserver
cntserver.on("cntserver:up", function (url) {
    log.info("Server up", url);
    updateUserName();
    startListenNotifications();
    checkRegisteredUser();
    setTimeout(popupUpdaterCfg, 1000);
    setTimeout(popupStartLogic(signIn), 1000 * 10)
});

cntserver.on("cntserver:down", function (url) {
    log.info("Server down");
    stopListenNotifications()
});

cntserver.on("cntserver:unauthorized", function (url) {
    log.info("Server unauthorized");
    setAuthorized(false);
    mainWindow.isVisible() && (mainWindow.hide() || popupWindow.show())
});

app.on('activate-with-no-open-windows', show);

mainWindow.onHidden(function () {
    checkSilent();
});

app.on('browser-window-closed', function () {
    checkSilent();
});

app.on("will-quit", function (event) {
    // Перед выходом пытаемся остановить cntserver
    if (!cntserver.dead()) {
        event.preventDefault();
        cntserver.stop()
            .then(function () {
                app.quit();
            })
            .catch(function () {
                // cntserver завершится по SIGING или Terminate на событие process.exit
                app.exit(0);
            });
    }
});


// Проверка наличия зарегистрированных девайсов (TODO: переделать на оповещение по сокету)
var check_registred = function () {
    var devices_up = true;
    cntserver.request("GET", "/devices")
        .then(function (data) {
            devices = data;
            for (var id in data) {
                if (data[id].registered) {
                    is_link = true;
                    break
                }
                if (data[id].registered === null) devices_up = false;
            }

            if (is_link) {
                log.info("PERMIT: Registered device. Done");
                setPermitConfig({"permited": true});
                mainWindow.show();
                mainWindow.focus();
                return
            }
            if (devices_up) {
                log.info("PERMIT: Registred device not exist. Show PermitWindow");
                permitWindow.show(device_id);
                return
            }
            setTimeout(check_registred, 1000 * 3)
        });
};


// Запрос к cntserver на авторизацию
// Должно всплыть окно браузера с открытым sso
// Вернет управление когда пользователь авторизуется
var signIn = function (type) {
    popupWindow.hide();
    return cntserver.request("POST", type ? util.format("/login/%s", type) : "/login")
        .then(function (data) {
            setAuthorized(true);
            log.info(util.format("is reg %s", data["is_registred"]));
            if (data["is_registred"] === "True") {
                cntserver.request("GET", "/devices")
                    .then(function (data) {
                        devices = data;
                        permitWindow.show(device_id)
                    })
            } else {
                if (is_link) {
                    mainWindow.show();
                    mainWindow.focus()
                } else {
                    check_registred();
                }
            }
        })
};


// Запрос на удаление авторизационных данных
var signOut = function () {
    return cntserver.request("POST", "/logout")
        .then(function () {
            setAuthorized(false);
        });
};


// Создать меню для текущего состояния приложения
var createMainMenu = function (without_devices) {
    var deviceItems = [];
    var makeDeviceItemHandler = function (deviceId) {
        var deviceInfo = devices[id];
        return function () {
            deviceWindow.show(deviceId, deviceInfo);
        };
    };

    for (var id in devices) {
        deviceItems.push({
            label: devices[id].name + " " + translate("settings"),
            click: makeDeviceItemHandler(id)
        });
    }

    // Используется только для компонентов браузера, потому что для ангуляра используется библиотека.
    function translate(marker) {
        return translations[appLang]["browser"][marker];
    }

    var languageSubmenuItems = [];
    var makeLangItemHandler = function (keyLang) {
        var langInfo = key;
        return function () {
            setAppLang(langInfo);
            appLang = langInfo
        }
    };

    for (var key in translations) {
        if (translations.hasOwnProperty(key)) {
            languageSubmenuItems.push({
                label: translations[key].label,
                type: 'radio',
                checked: key === appLang,
                click: makeLangItemHandler(key)
            });
        }
    }

    var base_menu = [
        { type: 'separator' },
        {
            label: translate('language'),
            submenu: languageSubmenuItems
        },
        {
            label: translate('writeToSupport'),
            click: function () { support.sendEmail(); }
        },
        authorized ?
            // В строчке signout указываем имя пользователя
            { label: translate('signout') + ' ' + userName, click: signOut } :
            { label: translate('signin'), click: signIn },
        { label: translate('exit'), id: 'exit', click: function () { app.quit(); } }
    ];
    var template = without_devices ? base_menu : deviceItems.concat(base_menu);
    return Menu.buildFromTemplate(template);
};

var ContextMenuTemplate = [
    { label: "Cut", accelerator: "CmdOrCtrl+X", selector: "cut:" },
    { label: "Copy", accelerator: "CmdOrCtrl+C", selector: "copy:" },
    { label: "Paste", accelerator: "CmdOrCtrl+V", selector: "paste:" },
    { label: "Select All", accelerator: "CmdOrCtrl+A", selector: "selectAll:" },
    { type: "separator" },
    { label: "Undo", accelerator: "CmdOrCtrl+Z", selector: "undo:" },
    { label: "Redo", accelerator: "Shift+CmdOrCtrl+Z", selector: "redo:" }
];

var createContextMenu = function () {
    return Menu.buildFromTemplate(ContextMenuTemplate);
};


// Обновить имя авторизованного пользователя
var updateUserName = function () {
    if (userName) {
        return;
    }
    cntserver.request("GET", "/user")
        .then(function (userInfo) {
            userName = userInfo.name;
        });
};


// Обновить стостояние авторизованности
var setAuthorized = function (value) {

    var generalCfgUpd = {"authorized": value};
    if (value) generalCfgUpd.registered = true;
    setGeneralCfg(generalCfgUpd);

    if (authorized === value) {
        return;
    }
    authorized = value;
    if (authorized) {
        popupWindow.hide();
        updateUserName();
        startListenNotifications();
    } else {
        userName = "";
        stopListenNotifications();
    }

    // Пинаем окно поиска на перерисовку
    mainWindow.update();
};


// Обработчик нотификаций от сервера
var notificationCallback = function (data) {
    var message = JSON.parse(data);

    var deviceUpdates = message.deviceUpdates;

    // Пока только девайсы
    if (deviceUpdates) {
        for (var id in deviceUpdates) {
            if (deviceUpdates[id]) {
                // TODO notifications
                devices[id] = deviceUpdates[id];
            } else {
                deviceWindow.close(id);
                delete devices[id];
            }
        }
    }
};


// Начать слушать нотификации от сервера по websocket'у
var startListenNotifications = function () {
    cntserver.listen("/ws")
        .then(function (ws) {
            notifySocket = ws;
            notifySocket.on("message", notificationCallback);
            notifySocket.on("close", startListenNotifications);
        })
        .then(function () {
            return cntserver.request("GET", "/devices");
        })
        .then(function (data) {
            devices = data;
        });
};


// Перестать слушать нотификации от сервера
var stopListenNotifications = function () {
    if (!!notifySocket) {
        notifySocket.close();
        notifySocket = null;
    }
};


// Проверка наступления состояния silent
var checkSilent = function () {
    if (BrowserWindow.getAllWindows().length > 1) {
        return
    }
    if (mainWindow.isVisible()) {
        return
    }

    // Если осталось только одно главное окно и его не видно -- то считаем
    // что наступло состояние silent
    for (var index in silentCallbacks) {
        silentCallbacks[index]();
    }
    silentCallbacks = [];
};

// Зарегистрирован или нет пользователь
function checkRegisteredUser() {
    var cfg = getGeneralCfg();
    var registered = cfg.registered;
    if (registered !== null) return;
    log.info("FINDO-APP. Func checkRegisteredUser(). Start ...");

    cntserver.request("GET", '/is-authorizes').then(function (data) {
        if (data.status === true) {
            setGeneralCfg({"registered": true});
            log.info("FINDO-APP. Func checkRegisteredUser(). Registered = True")
        } else if (data.status === false) {
            setGeneralCfg({"registered": false});
            log.info("FINDO-APP. Func checkRegisteredUser(). Registered = False")
        }
    });
    setTimeout(checkRegisteredUser, 1000 * 30)
}


//
// Трекинг события первого запуска
//
var iidConf = getAppUUID();
if (!iidConf.notify) {
    log.info("Event tracking: install");
    require("./tracking-notifier").notify("install")
}


// Старт приложения
var click_on_tray = function () {
    if (authorized) {
        if (is_link || is_link === null) {
            check_registred();
            show()
        } else {
            permitWindow.show(device_id)
        }
    } else {
        popupWindow.show()
    }
};

trayIcon = new Tray(path.join(imagesDir, "normal.png"));
trayUtil.setImage(trayIcon, path.join(imagesDir, "normal.png"));
trayIcon.on('clicked', click_on_tray);
//trayIcon.setContextMenu(createMainMenu());

// TODO: позволять пользователю задавать свое сочетание
if (!globalShortcut.register("ctrl+alt+f", show)) {
    log.warn("Can't register shortcut");
}

// Меню приложения, чтобы работали Cmd+C, Cmd+V и прочее
Menu.setApplicationMenu(Menu.buildFromTemplate(
    [{
        label: "Edit",
        submenu: ContextMenuTemplate
    }]
));

cntserver.start();
versionInfo.enableAutoUpdates();


// Публичные методы и переменные
module.exports.show = show;
module.exports.signIn = signIn;
module.exports.signOut = signOut;
module.exports.createMainMenu = createMainMenu;
module.exports.createContextMenu = createContextMenu;
module.exports.translations = translations;

module.exports.set_permited = function () {
    is_link = true;
    setPermitConfig({"permited": true});
};

module.exports.show_onboarding = function () {
    cntserver.request("GET", "/open-onboarding")
        .then(function (data) {
            log.info(util.format("PERMIT: Show onboarding. Status *%s*", data.status));
            if (data.status === "ok") {
                setPermitConfig({"show_onboarding": false})
            }
        })
};

module.exports.isAuthorized = function () {
    return authorized
};

// Вызвать метод когда пользователь перестанет взаимодействовать с приложением
module.exports.onceSilent = function (callback) {
    silentCallbacks.push(callback);
    checkSilent();
};
