var fs = require('fs');
var path = require("path");
var util = require('util');
var request = require('request');
var cron = require('node-cron');

var cntserver = require('./cntserver');

var popupWindow = require('./windows/popup');
var log = require('../log');
var configPath = require("../config/config").configPath;
var partner = require("../config/config").partner;
var getAppLang = require("../config/config").getAppLang;
var getGeneralCfg = require("../config/config").getGeneralCfg;
var getPopupTranslate = require("../config/config").getPopupTranslate;
var config = require('../config/config').configs;

//const
var POPUP_STAT_FILENAME = "popup_stat.json";
var POPUP_SCH_FILENAME = "popup_sch.json";
var POPUP_STAT_PATH = path.resolve(configPath, POPUP_STAT_FILENAME);
var POPUP_SCH_PATH = path.resolve(configPath, POPUP_SCH_FILENAME);
var TIMEOUT_CHECK_PDF_TRIGGER = 1000 * 60 * 3; // 3 min
var TIMEOUT_UPDATER_SCH_CFG = 1000 * 60 * 3; // 3 min

var popupStat = {};
var popupSch = {};
var cronObj = {};
var timerIdPdfTrigger = null;
var timerUpdateSchCfg = null;
var signIn = function () {};

function stop() {
    log.warn("POPUP. Func stop(). Stop popup logic");
    clearTimeout(timerIdPdfTrigger);
    timerIdPdfTrigger = null;

    for (var item in cronObj) {
        if (cronObj.hasOwnProperty(item)) {
            cronObj[item].destroy();
        }
    }
    cronObj = {};

    popupStat = {};
    popupSch = {};
}

function start(funcSignIn) {
    signIn = funcSignIn;
    log.warn("POPUP. Func start(). Starting popup logic ...");
    readStatCfg();
    readSchCfg();
    if (!popupSch.popups) {
        log.error("POPUP. Func start(). Failed, config is empty");
        stop();
        return false
    }
    if (!popupStat.general) {
        popupStat.general = {
            "count": 0,
            "showDate": null
        };
        writeStatCfg()
    }
    if (!popupStat.popups) {
        popupStat.popups = {};
        writeStatCfg()
    }


    for (var popupId in popupSch.popups) {
        if (popupSch.popups.hasOwnProperty(popupId)) {
            if (!popupStat.popups[popupId]) {
                popupStat.popups[popupId] = {
                    "count": 0,
                    "showDate": null
                };
                writeStatCfg()
            }
            var cronId = getCronTask(popupId);
            if (cronId) {
                log.info(util.format("POPUP. Func start(%s). Init popup task ...", popupId));
                cronId.start();
                cronObj[popupId] = cronId;
            }
        }
    }
}

// Остановка заданий крона на проверку попап у которых одно из условий гарантирует результат false
function destroyTask(popupId, reason) {
    if (popupId in cronObj) {
        log.info(util.format("POPUP. Func destroyTask(%s), reason *%s*", popupId, reason));
        cronObj[popupId].destroy()
    }
}

function getCronTask(popupId) {
    var cfg = popupSch.popups[popupId];
    if (!cfg) {
        log.warn(util.format("POPUP. Func getCronTask(%s). Failed, config is empty", popupId));
        return false
    }

    var valid = cron.validate(cfg.cron);
    if (!valid) {
        log.warn(util.format("POPUP. Func getCronTask(%s). Failed, cron string incorrect", popupId));
        return false
    }

    return cron.schedule(cfg.cron, function () {
        popupLogic(popupId)
    }, false)
}

function popupLogic(popupId) {
    var logic = popupSch.popups[popupId].logic;
    var destroyReason;
    if (!logic) {
        log.info(util.format("POPUP. Func popupLogic(%s). Logic is empty = true", popupId));
        showPopup(popupId);
    } else {

        var pass = true;
        var prop;
        var keyProperties = ["max", "partner"]; // проверяем ключевые условия

        for (var i = 0; i < keyProperties.length; i++) {
            prop = keyProperties[i];
            if (prop in logic) {
                if (logic.hasOwnProperty(prop)) {
                    checkProp(prop);
                    if (!pass) {
                        destroyReason = prop;
                        break
                    }
                }
            }
        }

        // проверяем оставшиеся условия
        for (prop in logic) {
            if (logic.hasOwnProperty(prop) && !include(keyProperties, prop)) {
                if (!pass) break;
                checkProp(prop)
            }
        }

        if (pass) {
            showPopup(popupId)
        } else {
            if (destroyReason !== undefined) destroyTask(popupId, destroyReason)
        }

    }

    function checkProp(prop) {
        switch(prop) {
            case "max":
                pass = isMax(popupId);
                log.info(util.format("POPUP. Func popupLogic(%s). Logic *max* = %s", popupId, pass));
                break;
            case "totalCount":
                pass = isTotalMax(popupId);
                log.info(util.format("POPUP. Func popupLogic(%s). Logic *totalCount* = %s", popupId, pass));
                break;
            case "partner":
                pass = isPartner(popupId);
                log.info(util.format("POPUP. Func popupLogic(%s). Logic *partner* = %s", popupId, pass));
                break;
            case "passedTime":
                pass = isPassedTime(popupId);
                log.info(util.format("POPUP. Func popupLogic(%s). Logic *passedTime* = %s", popupId, pass));
                break;
            case "language":
                pass = isLanguage(popupId);
                log.info(util.format("POPUP. Func popupLogic(%s). Logic *language* = %s", popupId, pass));
                break;
            case "registered":
                pass = isRegistered(popupId);
                log.info(util.format("POPUP. Func popupLogic(%s). Logic *registered* = %s", popupId, pass));
                break;
            case "authorized":
                pass = isAuthorized(popupId);
                log.info(util.format("POPUP. Func popupLogic(%s). Logic *authorized* = %s", popupId, pass));
                break;
            case "popup":
                pass = isPopup(popupId);
                log.info(util.format("POPUP. Func popupLogic(%s). Logic *popup* = %s", popupId, pass));
                break;
            case "pdfTrigger":
                pass = isPdfTrigger(popupId);
                log.info(util.format("POPUP. Func popupLogic(%s). Logic *pdfTrigger* = %s", popupId, pass));
                break;
        }
    }

    function include(arr, obj) {
        for(var i=0; i<arr.length; i++) {
            if (arr[i] == obj) return true;
        }
    }

}

function showPopup(popupId) {

    var count = 1;
    var time = 180;
    var series = popupSch.popups[popupId].series;

    var text = popupSch.popups[popupId].text;
    if (text) {
        var trigger = text.match( /^\{\{(.*)\}\}?/i )[1];
        if (trigger) {
            text = getPopupTranslate()[trigger] || getPopupTranslate()["regular"]
        }
    } else text = getPopupTranslate()["regular"];

    if (series) {
        count = series.count || count;
        time = series.time || time
    }

    log.info("POPUP. Func showPopup(%s). Count in series = %s, delay = %s (sec), text = /* %s */", popupId, count, time, text);

    setTimeout(show, 0, 1, text, popupId);

    function show(iter, message, popupId) {
        if (iter === 1) {
            var now = new Date();
            popupStat.general.count += 1;
            popupStat.general.showDate = now;
            popupStat.popups[popupId].count += 1;
            popupStat.popups[popupId].showDate = now;
            if (popupStat.popups[popupId].pdfTrigger) popupStat.popups[popupId].pdfTrigger = false;
            writeStatCfg()
        }
        log.info(util.format("POPUP. Func show(%s). Iter %s/%s", popupId, iter, count));
        if (require('./util/tools').isLessThanWin10()) {
            const WindowsBalloon = require('node-notifier').WindowsBalloon;
            var notifierBalloon = new WindowsBalloon({
                withFallback: false
            });
            var propBalloon = {
                title: 'Findo',
                message: message,
                time: 60000, // How long to show balloon in ms
                wait: true, // Wait for User Action against Notification
                type: 'info' // The notification type : info | warn | error
            };
            notifierBalloon.notify(propBalloon, function (error, response) {if (Boolean(error)) log.info(error)});
            require("./tracking-notifier").notify("showpopup_" + popupId);
            notifierBalloon.on('click', function (notifierObject, options) {
                log.info('POPUP. Click callback event');
                if (getAuthStatus()) {
                    signIn()
                } else {
                    popupWindow.show();
                    popupWindow.focus()
                }
                require("./tracking-notifier").notify("clickpopup_" + popupId)
            });
        } else {
            const notifier = require('node-notifier');
            var prop = {
                title: 'Findo',
                message: message,
                wait: true,
                timeout: 60,
                icon: path.join(__dirname, 'images/win32/normal@2.5x.png')
            };
            notifier.notify(prop, function (error, response) {if (Boolean(error)) log.info(error)});
            require("./tracking-notifier").notify("showpopup_" + popupId);
            notifier.on('click', function (notifierObject, options) {
                log.info('POPUP. Click callback event');
                if (getAuthStatus()) {
                    signIn()
                } else {
                    popupWindow.show();
                    popupWindow.focus()
                }
                require("./tracking-notifier").notify("clickpopup_" + popupId)
            });
        }
        if (iter < count) setTimeout(show, 1000 * time, ++iter, message, popupId)
    }

}

// logic
function isRegistered(popupId) {
    var prop = popupSch.popups[popupId].logic.registered.value;
    var registered = getGeneralCfg().registered;
    return prop === undefined || prop === registered
}

function isAuthorized(popupId) {
    var prop = popupSch.popups[popupId].logic.authorized.value;
    return prop === undefined || prop === getAuthStatus()
}

function getAuthStatus() {
    return getGeneralCfg().authorized
}

function isMax(popupId) {
    var count = 0;
    var prop = popupSch.popups[popupId].logic.max.value;

    var stat = popupStat.popups;
    if (stat) {
        if (popupId in stat) {
            var statPopup = stat[popupId];
            count = statPopup.count || 0
        }
    }

    return prop === undefined || count < prop
}

function isTotalMax(popupId) {
    var count = 0;

    var stat = popupStat.general;
    if (stat) {
        count = stat.count || 0;
    }

    var totalCount = popupSch.popups[popupId].logic.totalCount;
    return (totalCount.gt === undefined || totalCount.gt < count) &&
        (totalCount.lt === undefined || totalCount.lt > count) &&
        (totalCount.eq === undefined || totalCount.eq === count);

}

function isPassedTime(popupId) {

    var passedTime = 0;

    var stat = popupStat.general;
    if (stat) {
        if ("showDate" in stat) {
            passedTime = timeDeltaBySec(stat.showDate)
        }
    }

    var prop = popupSch.popups[popupId].logic.passedTime.value;

    return prop === undefined || prop < passedTime
}

function isPopup(popupId) {

    var popup = popupSch.popups[popupId].logic.popup;
    var id = popup.id;
    var stat = popupStat.popups;
    if (id === undefined || !(id in stat)) return true;

    var statPopup = stat[id];
    var count = statPopup.count || 0;
    var showDate = statPopup.showDate;
    var passedTime = timeDeltaBySec(showDate);

    var showed = popup.showed || {};
    var passed = popup.passed || {};

    // check showDate !== null
    if ((passed.lt || passed.gt) && (showDate === null)) return false;

    return (showed.lt === undefined || showed.lt > count) &&
        (showed.gt === undefined || showed.gt < count) &&
        (passed.lt === undefined || passed.lt > passedTime) &&
        (passed.gt === undefined || passed.gt < passedTime)

}

// Прошло секунд после показа
function timeDeltaBySec(date) {
    var now = new Date();
    var old = new Date(date);
    var timeDiff = Math.abs(now.getTime() - old.getTime());
    return Math.floor(timeDiff / 1000);
}

function isPartner(popupId) {

    function eq(elem) {
        return partner.toLowerCase() === elem.toLowerCase()
    }

    var popups = popupSch.popups;
    if (popupId in popups) {
        var popup = popups[popupId];
        if ("logic" in popup) {
            if ("partner" in popup.logic) {
                var partners =  popup.logic.partner.value || [];
                return partners.some(eq)
            }
        }
    }


    return true
}

function isLanguage(popupId) {
    var lang = getAppLang();
    function eq(elem) {
        return lang.toLowerCase() === elem.toLowerCase()
    }

    var popups = popupSch.popups;
    if (popups) {
        if (popupId in popups) {
            var popup = popups[popupId];
            if ("logic" in popup) {
                if ("language" in popup.logic) {
                    var language =  popup.logic.language.value || [];
                    return language.some(eq)
                }
            }
        }
    }

    return true
}

function isPdfTrigger(popupId) {
    var prop = popupSch.popups[popupId].logic.pdfTrigger.value;
    var stat = popupStat.popups[popupId];
    if (stat.hasOwnProperty("pdfTrigger")) {
        if (stat.pdfTrigger) {
            return prop === undefined || prop === true
        }
    } else {
        stat.pdfTrigger = false;
        writeStatCfg()
    }
    checkPdfTrigger();
    return prop === undefined || prop === false
}

function checkPdfTrigger() {
    if (timerIdPdfTrigger) return true;
    timerIdPdfTrigger = setTimeout(pdfTrigger, 0);

    function pdfTrigger() {
        for (var popupId in popupStat.popups) {
            if (popupStat.popups[popupId].hasOwnProperty("pdfTrigger")) {
                if (!popupStat.popups[popupId].pdfTrigger) {
                    cntserver.request("GET", "/pdf-trigger").
                    then(function (data) {
                        if (data.status === "trigger") {
                            setStatTriggerTrue();
                            writeStatCfg();
                            clearTimeout(timerIdPdfTrigger);
                            timerIdPdfTrigger = null
                        } else {
                            timerIdPdfTrigger = setTimeout(pdfTrigger, TIMEOUT_CHECK_PDF_TRIGGER);
                        }
                    });
                    break
                }
            }
        }

    }

    function setStatTriggerTrue() {
        for (var popupId in popupStat.popups) {
            if (popupStat.popups[popupId].hasOwnProperty("pdfTrigger")) {
                if (!popupStat.popups[popupId].pdfTrigger) {
                    popupStat.popups[popupId].pdfTrigger = true
                }
            }
        }
    }
}

/* rw config for status and scheduler popup*/
function writeStatCfg() {
    fs.writeFileSync(POPUP_STAT_PATH, JSON.stringify(popupStat), 'utf8');
}

function readStatCfg() {
    if (fs.existsSync(POPUP_STAT_PATH)) popupStat = JSON.parse(fs.readFileSync(POPUP_STAT_PATH, 'utf8'))
}

function writeSchCfg() {
    fs.writeFileSync(POPUP_SCH_PATH, JSON.stringify(popupSch), 'utf8');
}

function readSchCfg() {
    if (fs.existsSync(POPUP_SCH_PATH)) popupSch = JSON.parse(fs.readFileSync(POPUP_SCH_PATH, 'utf8'));
}

function runUpdaterSchCfg() {
    timerUpdateSchCfg = setTimeout(updaterSchCfg, 0);

    function updaterSchCfg() {
        var url = util.format("%s%s", config.cloud.cfgUrl, POPUP_SCH_FILENAME);
        var newPopupSch = {};
        var cfgData = "";
        log.info("POPUP. Func updaterSchCfg(). Try config update ...");

        request({method: 'GET', uri: url})
            .on('data', function(chunk) {
                cfgData += chunk
            })
            .on('end', function () {
                try {
                    newPopupSch = JSON.parse(cfgData);
                } catch(err) {
                    log.warn(util.format("POPUP (UpdaterSchCfg): %s", err))
                }
                if (newPopupSch && JSON.stringify(newPopupSch) !== JSON.stringify(popupSch)) {
                    popupSch = newPopupSch;
                    writeSchCfg();
                    log.warn("POPUP. Func updaterSchCfg(). Config success updated");
                    stop();
                    start()
                }
                log.info("POPUP. Func updaterSchCfg(). Config wasn`t change");
            })
            .on('error', function (error) {
                log.error("POPUP. Func updaterSchCfg(). Request error")
            });
        setTimeout(updaterSchCfg, TIMEOUT_UPDATER_SCH_CFG)
    }
}

module.exports.popupStartLogic = start;
module.exports.popupUpdaterCfg = runUpdaterSchCfg;