var shell = require("shell");
var url = require("url");
var fs = require("fs");
var os = require("os");
var path = require("path");
var childProcess = require("child_process");
var util = require("util"); // Node util
var manifest = require("../package.json");
var log = require("../log");


var makeTempDir = function(prefix) {
  var dirPath;
  while (true) {
    prefix = prefix ? prefix: "tmp";
    dirName = prefix + "-" + Math.random().toString(16).slice(2);
    dirPath = path.resolve(os.tmpdir(), dirName);
    try {
      fs.mkdirSync(dirPath);
      return dirPath;
    } catch (err) {
      if (err.code !== "EEXIST") {
        throw err;
      }
    }
  }
};


var collectInfo = function() {
  var tempDir = makeTempDir();
  // Отдельная папка для результата чтобы пользователь точно его ни с чем не перепутал
  var resultDir = makeTempDir(manifest.name);
  var processListFile = path.resolve(tempDir, "processes.txt");
  if (process.platform == "win32") {
    childProcess.execSync(util.format(
      'wmic process get processid,commandline /Format:texttable | find /i "%s" > "%s"',
      manifest.name,
      processListFile
    ));

    // приседания для создания cab файла
    var listToPack = path.resolve(tempDir, "list.txt");
    var resultName = "attach_this_file_please.cab";
    childProcess.execSync(util.format('copy %s\\* %s', log.logDir, tempDir));
    childProcess.execSync(util.format('dir /s /b "%s" > "%s"', tempDir, listToPack));
    childProcess.execSync(
      util.format(
        'makecab /F %s /D "CabinetNameTemplate=%s" /D "DiskDirectoryTemplate="',
        listToPack,
        resultName
      ),
      {cwd: tempDir}
    );
    childProcess.execSync(util.format(
      'move "%s" "%s"',
      path.resolve(tempDir, resultName),
      path.resolve(resultDir, resultName)
    ));
    return path.resolve(resultDir, resultName);
  } else {
    childProcess.execSync(util.format(
      'ps -A | grep "%s" > "%s"',
      manifest.name,
      processListFile
    ));
    var result = path.resolve(resultDir, "attach_this_file_please.tar.gz");
    childProcess.execSync(util.format(
      'tar -czf "%s" "%s" "%s"',
      result,
      log.logDir,
      processListFile
    ));
    return result;
  }
};


var mailtoTemplate = {
  protocol: "mailto",
  host: manifest.options.support.mail,
  query: {
    subject: util.format("%s v%s issue", manifest.name, manifest.version),
    body: ["IMPORTANT: Please attach the file from the window below before sending.",
           "",
           "",
           "Hi, Findo team!",
           "",
           "I encountered a problem using a desktop client.",
           "When the problem occured I was ...",
           "",
           "Please let me know when this issue is fixed.\n",
           "P.S. Please find attached logs for specifing the problem",
           ""
         ].join("\n")
  }
};


module.exports.sendEmail = function() {
  var collectedFile = collectInfo();
  var template = JSON.parse(JSON.stringify(mailtoTemplate));
  if (!!collectedFile) {
    template.query.attach = collectedFile;
  }
  shell.showItemInFolder(collectedFile);
  // Чтобы почта показалась выше окна с файлом
  setTimeout(function() {
    shell.openExternal(url.format(template));
  }, 1000);
};
