var path = require('path');
var fs = require('fs');

var currentDPI = null;
var dpiSuffix = {
  96: "",
  120: "@1.25x",
  144: "@1.5x",
  196: "@2x",
  240: "@2.5x",
  280: "@3x"
};

if (process.platform == "win32") {
  try {
    var result = require('child_process').execSync("reg query " +
      "\"HKEY_CURRENT_USER\\Control Panel\\Desktop\\WindowMetrics\" " +
      "/v AppliedDPI", {timeout: 1000}).toString();
    var match = result.match(/([\S]*)[^\S]*$/)
    if (match !== null) {
      currentDPI = parseInt(match[1]);
    }
  } catch(err) {
    console.log("Check DPI error:", err);
  }
}

var splitExt = function(filePath) {
  var extPos = filePath.lastIndexOf(".");
  if (extPos == -1) {
    return {base: filePath, ext:""};
  }
  return {base:filePath.slice(0, extPos), ext:filePath.slice(extPos)};
}

var selectBestResolution = function(filePath) {
  if (currentDPI !== null) {
    var split = splitExt(filePath)
    for (var dpi in dpiSuffix) {
      var dpiSpecificPath = split.base + dpiSuffix[dpi] + split.ext
      if (dpi >= currentDPI && fs.existsSync(dpiSpecificPath)) {
        return dpiSpecificPath;
      }
    }
  }
  return filePath;
}

if (process.platform == "darwin") {
  module.exports.setImage = function(tray, image) {
    var split = splitExt(image);
    var normalImage = split.base + "Template" + split.ext;
    var pressedImage = split.base + "Pressed" + split.ext;
    tray.setImage(fs.existsSync(normalImage) ? normalImage : image);
    tray.setPressedImage(fs.existsSync(pressedImage) ? pressedImage : image);
  }
} else {
  module.exports.setImage = function(tray, image) {
    tray.setImage(selectBestResolution(image));
  }
}
