var updater = require('webkit-updater');
var manifest = require('../package.json');
var log = require('../log');


module.exports.current = manifest.version;
module.exports.checkForUpdates = updater.checkForUpdates;
module.exports.installAndRestart = updater.installAndRestart;

var config = manifest.options.updates;
var autoUpdateTimer = null;
var autoUpdateEnabled = false;
var firstUpdateDelay = 1000 * 60 * config.firstAutoCheck;
var autoUpdateInterval = 1000 * 60 * config.nextAutoCheck;

module.exports.isAutoUpdatesEnabled = function() {
  return autoUpdateEnabled;
};


module.exports.enableAutoUpdates = function() {
  if (autoUpdateEnabled) {
    return;
  }
  autoUpdateEnabled = true;
  var firstUpdateInterval = Math.floor(firstUpdateDelay * Math.random());
  autoUpdateTimer = setTimeout(autoUpdate, firstUpdateInterval);
};

module.exports.disableAutoUpdates = function() {
  if (!autoUpdateEnabled) {
    return;
  }
  autoUpdateEnabled = false;
  if (autoUpdateTimer) {
    clearTimeout(autoUpdateTimer);
    autoUpdateTimer = null;
  }
};


var restartTimer = function() {
  if (autoUpdateEnabled && !autoUpdateTimer) {
    autoUpdateTimer = setTimeout(autoUpdate, autoUpdateInterval);
  }
};


var autoUpdate = function() {
  autoUpdateTimer = null;
  log.info("Autmatically check for updates");

  updater.checkForUpdates()
  .then(function(result) {
    if (!result) {
      restartTimer();
    } else {
      log.info("Update available %s", result);
      require('./findo-app').onceSilent(autoInstall);
    }
  })
  .catch(function(err) {
    log.warn("Update check error: %s", err);
    restartTimer();
  });
};

var autoInstall = function() {
  if (autoUpdateEnabled) {
    log.info("Will install and restart");
    updater.installAndRestart(["--silent"])
    .catch(function(err){
      log.warn("Install and restart error: %s", err);
      restartTimer();
    });
  }
};
