var app = require("app");
var fs = require('fs');
var path = require("path");
var util = require('util');
var lcid = require('lcid');
var childProcess = require("child_process");

var configs = require('./partners.json');

var log = require('../log');
var packageConfigs = require('../package.json');
var translations = require('../browser/translations');
var updateObject = require('../browser/util/tools').updateObject;
var isEmptyObject = require('../browser/util/tools').isEmptyObject;

// const
var CONFIG_DIR = 'config';
var GENERAL_CONFIG = 'general.json';
var LANG_CONFIG = 'lang.json';
var PERMIT_CONFIG = 'permit_cfg.json';
var UUID_APP_FILE = 'iid.json';

// Получить корневую директорию приложения
var rootPath = getRootPath();

function getRootPath() {
    var appName = packageConfigs.name;
    var rootPath = '';

    switch (process.platform) {
        case 'win32':
            // %LOCALAPPDATA%/Findo
            rootPath = path.resolve(app.getPath('home'), 'AppData', 'Local', appName);
            break;
        case 'darwin':
            // ~/Library/Findo
            rootPath = path.resolve(app.getPath('home'), 'Library', 'Application Support', appName);
            break;
    }

    return rootPath;
}

// Получение партнера
function getPartner () {
    var partnerMarkPath = path.resolve(rootPath, 'vendor.json');

    var partner = 'default';
    var predefinedPartner = packageConfigs.options.partner.trim();

    if  (fs.existsSync(partnerMarkPath) && predefinedPartner === "None") {
        var partnerMark = JSON.parse(fs.readFileSync(partnerMarkPath, 'utf8'));
        partner = partnerMark.partner.trim();
    } else if (!fs.existsSync(partnerMarkPath) && predefinedPartner !== "None") {
        fs.writeFileSync(partnerMarkPath, util.format('{ \"partner\" : \"%s\" }', predefinedPartner), 'utf8');
        partner = predefinedPartner;
    } else if (predefinedPartner !== "None") {
        partner = predefinedPartner;
    }

    return partner
}

// Получение конфигурации для партнера
function getConfigs() {
    var partner = getPartner();
    if (partner && configs[partner]) {
        return configs[partner];
    }
    return configs['default'];
}

var partnerConfigs = getConfigs();

// Получение общих данных (регистрация, авторизация)
var generalConfig = {};

function getGeneralCfg() {
    if (!isEmptyObject(generalConfig)) return generalConfig;
    var generalCfgPath = path.resolve(rootPath, CONFIG_DIR, GENERAL_CONFIG);
    if  (fs.existsSync(generalCfgPath)) {
        generalConfig = JSON.parse(fs.readFileSync(generalCfgPath, 'utf8'));
    } else {
        generalConfig = {
            "registered": null,
            "authorized": true
        };
        fs.writeFileSync(generalCfgPath, JSON.stringify(generalConfig), 'utf8')
    }
    return generalConfig
}

function setGeneralCfg(data) {
    updateObject(generalConfig, data);
    var generalCfgPath = path.resolve(rootPath, CONFIG_DIR, GENERAL_CONFIG);
    fs.writeFileSync(generalCfgPath, JSON.stringify(generalConfig), 'utf8')
}


// Получение языка интерфейса
var appLang;

function getAppLang() {

    if (appLang !== undefined) return appLang;

    var defaultLang = 'en';
    var langConfig = path.resolve(rootPath, CONFIG_DIR, LANG_CONFIG);

    if  (fs.existsSync(langConfig)) {
        var appLangConfig = JSON.parse(fs.readFileSync(langConfig, 'utf8'));
        appLang = appLangConfig.language.trim() || defaultLang;
    } else {
        appLang = getSystemLocale() || defaultLang;
        fs.writeFileSync(langConfig, util.format('{ \"language\" : \"%s\" }', appLang), 'utf8');
    }

    return appLang
}

var setAppLang = function (lang) {
    appLang = lang;
    var langConfig = path.resolve(rootPath, CONFIG_DIR, LANG_CONFIG);
    fs.writeFileSync(langConfig, util.format('{ \"language\" : \"%s\" }', lang), 'utf8');
};

// Плучение локали системы
function getSystemLocale() {
    var com_lang = Object.keys(translations.translations);
    var lang_os;
    var lang = '';

    function getWinLocaleSync() {
        var res = childProcess.execSync('wmic os get locale');
        const lcidCode = parseInt(res.toString().replace('Locale', ''), 16);
        return lcid.from(lcidCode);
    }

    function getAppleLocaleSync() {
        var res = childProcess.execSync('defaults read -g AppleLocale');
        return res.toString();
    }

    switch (process.platform) {
        case 'win32':
            lang_os = getWinLocaleSync();
            break;
        case 'darwin':
            lang_os = getAppleLocaleSync();
            break;
    }

    lang_os = lang_os.replace("-", "_").toLowerCase().trim();
    com_lang.forEach(function (lang_item) {
        if (lang_item.toLowerCase().trim() === lang_os) {
            lang = lang_item;
        }
    });

    if (!lang) {
        com_lang.forEach(function (lang_item) {
            if (lang_item.toLowerCase().trim().slice(0, 2) === lang_os.slice(0, 2)) {
                lang = lang_item
            }
        })
    }

    return lang;
}

// Конфигурация разрешения на индексацию
getPermitConfig = function () {
    var permit = path.resolve(rootPath, CONFIG_DIR, PERMIT_CONFIG);
    if  (fs.existsSync(permit)) {
        var permit_cfg = JSON.parse(fs.readFileSync(permit, 'utf8'));
        return permit_cfg;
    } else {
        var default_conf = {
            "permited": false,
            "show_onboarding": true
        };
        fs.writeFileSync(permit, JSON.stringify(default_conf), 'utf8');
        // признак того, что произошло обновление на версию с разрешением на индексацию
        default_conf.permited = null;
        return default_conf;
    }
};

setPermitConfig = function (data) {

    var permit = path.resolve(rootPath, CONFIG_DIR, PERMIT_CONFIG);
    if  (fs.existsSync(permit)) {
        var permit_cfg = JSON.parse(fs.readFileSync(permit, 'utf8'));
        var new_cfg = updateObject(permit_cfg, data);
        fs.writeFileSync(permit, JSON.stringify(new_cfg), 'utf8');
        return new_cfg;
    } else {
        var default_conf = {
            "permited": false,
            "show_onboarding": true
        };
        var new_cfg = updateObject(default_conf, data);
        fs.writeFileSync(permit, JSON.stringify(new_cfg), 'utf8');
        return new_cfg;
    }
};

// локализация попап
getPopupTranslate = function () {
    var translate = require('../browser/popup_translations.json');
    return translate[getPartner()][getAppLang()]
};

// UUID установленного приложения
getAppUUID = function () {
    const uuidV4 = require('uuid/v4');
    var now = new Date().toISOString();
    var uuidPath = path.resolve(rootPath, CONFIG_DIR, UUID_APP_FILE);
    if  (fs.existsSync(uuidPath)) {
        var uuidApp = JSON.parse(fs.readFileSync(uuidPath, 'utf8'));
        return uuidApp;
    } else {
        var new_uuid = uuidV4();
        log.info("Set IID(UUID) for app " + new_uuid);
        var new_data = {
            "iid": new_uuid,
            "iidcreateat": now,
            "notify": false
        };
        fs.writeFileSync(uuidPath, JSON.stringify(new_data), 'utf8');
        return new_data;
    }
};

setAppUUIDNotifier = function (flag) {
    var uuidPath = path.resolve(rootPath, CONFIG_DIR, UUID_APP_FILE);
    if  (fs.existsSync(uuidPath)) {
        var uuidApp = JSON.parse(fs.readFileSync(uuidPath, 'utf8'));
        uuidApp.notify = flag;
        fs.writeFileSync(uuidPath, JSON.stringify(uuidApp), 'utf8');
        return uuidApp
    }
};

var makeDirs = function(dirPath) {
    try {
        fs.mkdirSync(dirPath);
    } catch (err) {
        switch (err.code) {
            // Не падаем если директория уже есть
            case "EEXIST":
                return;
            // Создаем родительскую директорию если ее нет
            case "ENOENT":
                makeDirs(path.dirname(dirPath));
                return fs.mkdirSync(dirPath);
            default:
                throw err;
        }
    }
};

var moveToConfigDir = function (file) {
    var old_file = path.resolve(rootPath, file);
    var new_file = path.resolve(rootPath, CONFIG_DIR, file);

    if  (fs.existsSync(old_file)) {
        fs.renameSync(old_file, new_file)
    }
};

// Переносим все старые конфиги в новую папку
module.exports.moveCfg = function () {
    makeDirs(path.resolve(rootPath, CONFIG_DIR));
    moveToConfigDir(UUID_APP_FILE);
    moveToConfigDir(LANG_CONFIG);
    moveToConfigDir(PERMIT_CONFIG);
};

// Экспорт
module.exports.rootPath = rootPath;
module.exports.configPath = path.resolve(rootPath, CONFIG_DIR);
module.exports.getAppLang = getAppLang;
module.exports.setAppLang = setAppLang;
module.exports.configs = partnerConfigs;
module.exports.getPopupTranslate = getPopupTranslate;
module.exports.getAppUUID = getAppUUID;
module.exports.setAppUUIDNotifier = setAppUUIDNotifier;
module.exports.setPermitConfig = setPermitConfig;
module.exports.getPermitConfig = getPermitConfig;
module.exports.partner = getPartner();
module.exports.getGeneralCfg = getGeneralCfg;
module.exports.setGeneralCfg = setGeneralCfg;