var app = require("app");
var updater = require("webkit-updater");
var ionisator = require("ionisator");

var config = require("./package.json").options;
var log = require("./log");
var firstRun = false;
var findoApp;

// Чтобы удобно было тестить из командной строчки
process.on("SIGINT", function() { process.exit(); });
process.on("SIGTERM", function() { process.exit(); });

app.on('ready', function() {
    // Инициализируем апдейтер
    updater.init(process.argv)
        .then(function(argv) {
            // Апдейтер вернул управление, значит это обычный запуск приложения

            // перемещаем старые конфиги в отдельную дирректорию
            require("./config/config").moveCfg();

            // Запускаем приложение через ionisator
            ionisator.init(argv, start, activate);
        }).catch(function(err) {
        console.log(err);
    });
});

// Зовется ionisator'ом если приложение еще не запущено
var start = function() {
    updater.setFeedUrl(config.updates.feedUrl);
    findoApp = require("./browser/findo-app");
    if (firstRun) {
        firstRun = false;
        findoApp.signOut();
    }
};

// Зовется ionisator'ом после старта
// или в ответ на попытку запустить приложение повторно
var activate = function(options) {
    log.info("Activation '%s'", options);
    var show = true;
    for (var argIndex in options) {
        // Если запустили с ключом silent то тихо живем в трее
        switch (options[argIndex]) {
            case "--silent":
                show = false;
                break;
            case "--exit":
                process.exit();
                return;
        }
    }
    if (options[0] != process.execPath && !!updater.restartLatestVersion) {
        log.info("Try run newest version");
        updater.restartLatestVersion(options.slice(1));
    } else if (show) {
        findoApp.show();
    }
};

// Кастомное дейтвие при установке
// Зовется апдейтером при первом запуске после установки
updater.handle('firstrun', function(resolve, reject) {
    firstRun = true;
    resolve();
});

// Кастомное дейтвие при установке
// Зовется апдейтером в момент установки приложения
updater.handle('install', function(resolve, reject) {
    updater.util.putInAutoRun(["--silent"])
        .then(function() {
            if (process.platform == "win32") {
                return updater.util.createDefaultShortcuts()
                    .then(updater.util.updateUninstallerIcon);
            }
        })
        .then(resolve, reject);
});

updater.handle('update', function(resolve, reject) {
    (process.platform == 'win32' ?
            updater.util.updateUninstallerIcon() : Promise.resolve()
    )
        .catch(function() {})
        .then(updater.util.lookupAutoRun)
        .then(function(autorun_info) {
            if (!autorun_info ||autorun_info.startArguments.length != 1 ||
                autorun_info.startArguments[0] != "--silent")
            {
                return updater.util.putInAutoRun(["--silent"]);
            }
        })
        .then(resolve, reject);
});

// Кастомное дейтвие при удалении
// Зовется апдейтером в момент удаления приложения
updater.handle('uninstall', function (resolve, reject) {
    var finallyAction = function() {
        try {
            // Шлем уже запущенному приложению запрос на закрытие
            ionisator.init(["--exit"], function() {}, function() {});
        } catch (err) {
            log.warn("Can't request app exit", err);
        }
        log.info("Uninstall app ...");
        require("./browser/tracking-notifier").notify("uninstall");
        return Promise.resolve();
    };

    updater.util.removeFromAutoRun()
        .then(finallyAction, finallyAction)
        .then(resolve, reject);
});
