var app = require("app");
var path = require("path");
var fs = require("fs");
var winston = require("winston");
const winston_daily_rotate_file = require('winston-daily-rotate-file');
var appName = require('./package.json').name;

// Находим подходящее место для лога на этой системе (подсмотрено в appdirs)
var logDir;
switch (process.platform) {
  case "win32":
    // %LOCALAPPDATA%/Findo/Logs
    logDir = path.resolve(app.getPath("home"), "AppData", "Local", appName, "Logs");
    break;
  case "darwin":
    // ~/Library/Logs/Findo
    logDir = path.resolve(app.getPath("home"), "Library", "Logs", appName);
    break;
  default:
    // ~/.cache/Findo/log
    logDir = path.resolve(app.getPath("home"), ".cache", appName, "log");
}


var makeDirs = function(dirPath) {
  try {
    fs.mkdirSync(dirPath);
  } catch (err) {
    switch (err.code) {
      // Не падаем если директория уже есть
      case "EEXIST":
        return;
      // Создаем родительскую директорию если ее нет
      case "ENOENT":
        makeDirs(path.dirname(dirPath));
        return fs.mkdirSync(dirPath);
      default:
        throw err;
    }
  }
};


// В мануалах winston сказано что созданием директории должен заниматься пользовательский код
makeDirs(logDir);

// Функция-генератор timestamp в ISO формате с локальным временем
const timestamp_generator = function() {
  const pad = function (number) {
    if (number < 10) {
      return '0' + number;
    }
    return number;
  };
  const date = new Date();
  return date.getFullYear() +
    '-' + pad(date.getMonth() + 1) +
    '-' + pad(date.getDate()) +
    ' ' + pad(date.getHours()) +
    ':' + pad(date.getMinutes()) +
    ':' + pad(date.getSeconds()) +
    ',' + (date.getMilliseconds() / 1000).toFixed(3).slice(2, 5);
};

var logger = new (winston.Logger)({
  transports: [
    new (winston.transports.Console)({
      timestamp: timestamp_generator
    }),
    new (winston_daily_rotate_file)({
      // filename без .log, ротатор файлов приклеит дату и расширение
      filename: path.resolve(logDir, "GUI"),
      // winston по умолчанию не ставит верменную метку
      timestamp: timestamp_generator,
      // читать будем глазами
      // решим использовать анализатор логов -- будем писать в json
      json: false,
      // Максимум кол-во файлов
      maxFiles: 10,
      // Шаблон даты по-умолчанию оставляет файлы без расширения
      datePattern: '.yyyy-MM-dd.log'
    })
  ]
});


// Пользоваться примерно так:
// var log = require('log');
//  log.info(...); log.warn(...); log.error(...)
module.exports = logger;

module.exports.logDir = logDir;
