var fs = require("fs");
var net = require('net');
var os = require('os');
var path = require('path');

var app = require("app");
var util = require("./util");

var getAppName = function() {
  var searchLocations = [path.join(process.resourcesPath, "app"), app.getAppPath()];
  for (i in searchLocations) {
    var manifest = util.safeRequire(path.join(searchLocations[i], "package.json"));
    if (manifest) {
      return manifest.name;
    }
  }
  return app.getName();
}


var getChannelName = function() {
  if(process.platform == "win32") {
    return path.join("\\\\.\\pipe", getAppName(), "ionisator_pipe");
  } else {
    return path.join(os.tmpdir(), getAppName() + "-#" + process.env.USER + "-ionisator.sock");
  }
}

// Listen for start of another app instance
var listenActivations = function(channelName, activateCallback) {
  var server = net.createServer();

  if(!channelName.match(/^\\\\[\.\?]\\pipe/)) {
    util.safeRemove(channelName);
    process.on("exit", function() {
      util.safeRemove(channelName);
      server.close();
    });
  }

  server
  .on('connection', function(connection) {
    connection.on('data', function(raw) {
      activateCallback(JSON.parse(raw));
      connection.destroy();
    });
  })
  .on('error', function(err) {
    console.error("Ionisator server error", err);
  });
  server.listen(channelName);
};


module.exports.init = function(options, start, activate) {
  var channelName = getChannelName();
  console.log(channelName);
  var startApp = function() {
    start();
    activate(options);
    listenActivations(channelName, activate);
  };

  try {
    var socket = net.connect({path: channelName});
    socket
    .once("connect", function() {
      try {
        socket.write(JSON.stringify(options), function() {
          socket.end();
          app.quit();
        });
      } catch (err) {
        console.error("Can't send optons", err);
      }
    })
    .once("error", function() {
      startApp();
      socket = null;
    });
  } catch (err) {
    console.log("Can't connect to channel. Starting app");
    startApp();
  }
};
