var EventEmitter = require("events").EventEmitter;
var util = require('./util');

var updater = require("../native/darwin/updater").updater;
var application_tools = require("../application-tools");

updater.__proto__ = EventEmitter.prototype;

var updateAvailable = null;

var formatError = function(err) {
  if ("NSLocalizedFailureReason" in err) {
    return err.NSLocalizedFailureReason;
  }
  return err;
};

module.exports.setFeedUrl = function(feedUrl) {
  updater.configure({
    feedUrl: feedUrl,
    appPath: application_tools.applicationRoot()
  });
};

module.exports.checkForUpdates = function () {
  return new Promise(function (resolve, reject) {
    if(updateAvailable) {
      return resolve(updateAvailable);
    }

    var onUpdateError = function(err) {
      console.error("Update error", err);
      reject(formatError(err));
      cleanup();
    };

    var onAvailable = function(item) {
      console.log('Update available', item);
      updateAvailable = item;
      resolve(item);
      cleanup();
    };

    var onUnavailable = function() {
      console.log("Update Unavailable");
      updateAvailable = null;
      resolve(null);
      cleanup();
    };

    updater.on('updateError', onUpdateError);
    updater.on('updateAvailable', onAvailable);
    updater.on('updateUnavailable', onUnavailable);

    var cleanup = function() {
      updater.removeListener("updateError", onUpdateError);
      updater.removeListener("updateAvailable", onAvailable);
      updater.removeListener("updateUnavailable", onUnavailable);
    };

    updater.checkForUpdates();
  });
};

module.exports.installAndRestart = function (restartArguments) {
  if (!updateAvailable) {
    return Promise.reject('Update Unavailable');
  }

  return new Promise(function (resolve, reject) {
    var onUpdateError = function(err) {
      console.error("Install error", err);
      reject(formatError(err));
      cleanup();
    };

    updater.on("updateError", onUpdateError);
    var cleanup = function() {
      updater.removeListener("updateError", onUpdateError);
      updateAvailable = null;
    };
    updater.installAndRestart(
      restartArguments.concat([util.updateSwitch]),
      function (err) {
        reject(err);
        cleanup();
      }
    );
  });
};
