var fs = require("fs");
var path = require("path");
var plist = require('plist');

var applicationTools = require("../application-tools");
var subprocess = require("../subprocess");

var getAppPlist = function() {
  var plist_file = path.resolve(
    applicationTools.applicationRoot(), "Contents", "Info.plist");

  return plist.parse(fs.readFileSync(plist_file, "utf8"));
};

var ensureDirExists = function(filePath) {
  try {
    fs.mkdirSync(path.dirname(filePath));
  } catch(e) {
    if (e.code != "EEXIST") {
      throw e;
    }
  }
};

var launchAgentPathFor = function(bundleId) {
  return path.resolve(
    process.env.HOME,
    "Library",
    "LaunchAgents",
    bundleId + ".plist"
  );
};

var updateSwitch = "--wku-update";
exports.updateSwitch = updateSwitch;

exports.parseStartupInfo = function(argv) {
  var isUpdate = false;
  var filterUpdateSwitch = function(arg) {
    if (arg != updateSwitch) {
      return true;
    }
    isUpdate = true;
    return false;
  };

  var result = {
    updaterEvents: [],
    applicationArgs: argv.filter(filterUpdateSwitch),
    shouldExit: false
  };

  var appPlist = getAppPlist();
  if (appPlist.WKUEmulateInstall) {
    var runStampPath = path.resolve(
      process.env.HOME,
      "Library",
      "Application Support",
      appPlist.CFBundleName,
      ".run-stamp"
    );

    var appRoot = applicationTools.applicationRoot();
    var runStamp = {
        location: appRoot,
        ctime: fs.statSync(applicationTools.applicationRoot()).ctime.getTime()
    };

    var prevRunStamp = null;
    try {
      prevRunStamp = JSON.parse(fs.readFileSync(runStampPath, "utf8"));
    } catch (error) {
      if (error.code != "ENOENT") {
        console.warn("Parse install timetsamp error", error);
      }
    }
    if (!prevRunStamp || prevRunStamp.location != runStamp.location) {
      // First launch from this place
      if (!prevRunStamp || prevRunStamp.ctime != runStamp.ctime) {
        // New app
        result.updaterEvents = ['install', 'firstrun'];
      } else {
        // Moved app
        result.updaterEvents = ['update'];
      }
    } else if (isUpdate) {
      // Launch from sparkle
      result.updateEvents = ['update'];
    } else if (prevRunStamp.ctime != runStamp.ctime) {
      // Lauch after manual update (may be app replaced from dmg image)
      result.updaterEvents = ['install', 'firstrun'];
    } else {
      // Subsequent launch
      return result;
    }
    // Should store new stamp
    ensureDirExists(runStampPath);
    fs.writeFileSync(runStampPath, JSON.stringify(runStamp), 'utf8');
  }
  return result;
};


exports.putInAutoRun = function(startArguments) {
  var appPlist = getAppPlist();
  var executable = path.resolve(
    applicationTools.applicationRoot(),
    "Contents",
    "MacOS",
    appPlist.CFBundleExecutable);

  var autorunPlist = {
    Label: appPlist.CFBundleIdentifier,
    ProgramArguments: [executable].concat(startArguments),
    ProcessType: 'Adaptive',
    RunAtLoad: true
  };

  var autorunEntryFile = launchAgentPathFor(appPlist.CFBundleIdentifier);

  return Promise.resolve()
  .then(function() {
    ensureDirExists(autorunEntryFile);
    fs.writeFileSync(autorunEntryFile, plist.build(autorunPlist));
  });
};


exports.lookupAutoRun = function() {
  var appPlist = getAppPlist();
  var executable = path.resolve(
    applicationTools.applicationRoot(),
    "Contents",
    "MacOS",
    appPlist.CFBundleExecutable);

  var autorunEntryFile = launchAgentPathFor(appPlist.CFBundleIdentifier);
  if (!fs.existsSync(autorunEntryFile)) {
    return Promise.resolve(null);
  }
  try {
    autorunEntry = plist.parse(fs.readFileSync(autorunEntryFile, "utf8"));
    var autorunExecutable = autorunEntry.ProgramArguments.slice(0, 1);
    if (autorunExecutable == executable) {
      return Promise.resolve(
        {startArguments: autorunEntry.ProgramArguments.slice(1)}
      );
    } else {
      console.warn("Different executable in autorun", autorunExecutable);
    }
  } catch(e) {
    console.warn("Launch Agents plist parsing error", e);
  }
  return Promise.resolve(null);
};


exports.removeFromAutoRun = function() {
  var autorunEntryFile = launchAgentPathFor(getAppPlist().CFBundleIdentifier);

  return Promise.resolve()
  .then(function() {
    if (fs.existsSync(autorunEntryFile)) {
      fs.unlinkSync(autorunEntryFile);
    }
  });
};
