var applicationTools = require("./application-tools");

var handlers = {
	install: [],
	uninstall: [],
	firstrun: [],
	update: []
};

var platformUtil = require("./" + process.platform + "/util");

module.exports = require("./" + process.platform + "/updater");
module.exports.util = {
	putInAutoRun: platformUtil.putInAutoRun,
	lookupAutoRun: platformUtil.lookupAutoRun,
	removeFromAutoRun: platformUtil.removeFromAutoRun,
	createDefaultShortcuts: platformUtil.createDefaultShortcuts,
	updateUninstallerIcon:  platformUtil.updateUninstallerIcon
};

module.exports.handle = function(updaterEvent, promiseResolver) {
	handlers[updaterEvent].push(promiseResolver);
};

module.exports.init = function(argv) {
  var startupInfo = platformUtil.parseStartupInfo(argv);

	var startup = Promise.resolve();
	for (var eventIndex in startupInfo.updaterEvents) {
		var event = startupInfo.updaterEvents[eventIndex];

		// Run command specific handlers
		var handlersChain = [];
		for (var handlerIndex in handlers[event]) {
			handlersChain.push(new Promise(handlers[event][handlerIndex]));
		}
		if(!handlersChain.length) {
			continue;
		}
		startup = startup.then(function() {
			return Promise.all(handlersChain);
		});
	}

	// Resolve with filtered command line args
	return startup.then(
	function() {
		if (startupInfo.shouldExit) {
			applicationTools.quit();
			return Promise.reject("Exit This");
		} else {
			return Promise.resolve(startupInfo.applicationArgs);
		}
	},
	function(err) {
		if (startupInfo.shouldExit) {
			applicationTools.quit();
		}
		return Promise.reject(err);
	});
};
