#ifndef WEBKIT_UPDATER_SRC_COCOA_BIND_H_
#define WEBKIT_UPDATER_SRC_COCOA_BIND_H_

#include <v8.h>
#include <native_mate/converter.h>


namespace mate {

template<>
struct Converter<NSString*> {
  static v8::Handle<v8::Value> ToV8(v8::Isolate* isolate, NSString* val) {
    if(val == nil) {
  		return MATE_UNDEFINED(isolate);	
  	}
    return ConvertToV8(isolate, std::string([val UTF8String]));
  }
};

template<>
struct Converter<NSURL*> {
  static v8::Handle<v8::Value> ToV8(v8::Isolate* isolate, NSURL* val) {
    if(val == nil) {
  		return MATE_UNDEFINED(isolate);	
  	}
    return ConvertToV8(isolate, [val absoluteString]);
  }
};

template<>
struct Converter<NSDate*> {
  static v8::Handle<v8::Value> ToV8(v8::Isolate* isolate, NSDate* val) {
  	if(val == nil) {
  		return MATE_UNDEFINED(isolate);	
  	}
  	return ConvertToV8(isolate, [val descriptionWithLocale:nil]);
  }
};

#define ID_COVERSION_MAP_BEGIN(isolate) if(false) { return MATE_UNDEFINED(isolate); } else 
#define ID_COVERSION_MAP_ENTRY(obj, isolate, cls) \
	if([(obj) isKindOfClass:[cls class]]) { return ConvertToV8(isolate, (cls*)(obj)); } else
#define ID_COVERSION_MAP_END(isolate) { return MATE_UNDEFINED((isolate)); }

static v8::Handle<v8::Value> convertId(v8::Isolate* isolate, id obj) {
	if(obj == nil) {
  		return MATE_UNDEFINED(isolate);	
  	}

	ID_COVERSION_MAP_BEGIN(isolate)
	ID_COVERSION_MAP_ENTRY(obj, isolate, NSString)
	ID_COVERSION_MAP_ENTRY(obj, isolate, NSURL)
	ID_COVERSION_MAP_ENTRY(obj, isolate, NSDate)
	ID_COVERSION_MAP_ENTRY(obj, isolate, NSArray)   	
	ID_COVERSION_MAP_ENTRY(obj, isolate, NSDictionary)
	ID_COVERSION_MAP_END(isolate)
}

template<>
struct Converter<NSArray*> {
  static v8::Handle<v8::Value> ToV8(v8::Isolate* isolate, NSArray* arr) {
    if(arr == nil) {
    	return MATE_UNDEFINED(isolate);		
    }
    
    v8::Handle<v8::Array> array = MATE_ARRAY_NEW(isolate, arr.count);
    int index = 0;
    for (id obj in arr) {
		  array->Set(index++, convertId(isolate, obj));
    }
    return array.As<v8::Value>();
  }
};


template<>
struct Converter<NSDictionary*> {
  static v8::Handle<v8::Value> ToV8(v8::Isolate* isolate, NSDictionary* dict) {
  	if(dict == nil) {
  		return MATE_UNDEFINED((isolate));	
  	}
    v8::Handle<v8::Object> dictionary = MATE_OBJECT_NEW(isolate);
    for(id key in dict) {
    	dictionary->Set(convertId(isolate, key), convertId(isolate, [dict objectForKey:key]));
    }
    return dictionary.As<v8::Value>();
  }
};
}
// namespace mate

#endif