var path = require("path");
var fs = require("fs");

var applicationTools = require("../application-tools");
var subprocess = require("../subprocess");

var squirrelPath = path.resolve(applicationTools.applicationRoot(), "Update.exe");
if(fs.existsSync(squirrelPath)) {
  module.exports.path = squirrelPath;
} else {
  squirrelPath = null;
  console.warn("Update.exe not found: " + squirrelPath);
}

var retry = function(runFunc, args, attemptsLeft) {
  return runFunc(args)
  .catch(function(err) {
    if (!attemptsLeft) {
      console.warn("Run squirrel error, no more attempts", args, err);
      return Promise.reject(err);
    }
    console.warn("Run squirrel error, will retry", args, err);
    return retry(runFunc, args, attemptsLeft - 1);
  });
}

module.exports.run = function(args, retryAttempts) {
  if (!squirrelPath) {
    return Promise.reject("Update.exe not found");
  } else if (!retryAttempts) {
    return subprocess.run([squirrelPath].concat(args));
  } else {
    return retry(subprocess.spawn, [squirrelPath].concat(args), retryAttempts);
  }

};

module.exports.spawn = function(args, retryAttempts) {
  if (!squirrelPath) {
    return Promise.reject("Update.exe not found");
  } else if (!retryAttempts) {
    return subprocess.spawn([squirrelPath].concat(args));
  } else {
    return retry(subprocess.spawn, [squirrelPath].concat(args), retryAttempts);
  }
};
