var path = require("path");

var applicationTools = require("../application-tools");
var util = require("./util");
var squirrel = require("./squirrel");

var feedUrl = "";
var updateInProgress = false;

module.exports.setFeedUrl = function(newFeedUrl) {
    feedUrl = newFeedUrl;
};

module.exports.checkForUpdates = function() {
    return squirrel.run(["--download", feedUrl, "--secureUpdates"])
        .then(function(output) {
            console.log("Update Available", output);
            // expect {currentVerison:..., futureVersion:..., releasesToApply:...}
            var statusString = output.trim().split('\n').pop();
            var lastRelease = JSON.parse(statusString).releasesToApply.pop();
            return Promise.resolve(lastRelease);
        })
        .catch(function(error) {
            console.log("Update Error", error);
            return Promise.reject(error);
        });
};

var restartNewVersion = function(restartParams) {
    var params = ["--processStartAndWait", path.basename(process.execPath)];
    if (restartParams.length) {
        params = params.concat(["-a", util.formatArgs(restartParams)]);
    }

    // C первой попытки заводится не всегда. Update.exe может возвращать EBUSY
    return squirrel.spawn(params, 5 /*retry attempts*/)
        .then(function() {
            applicationTools.quit();
        });
};

module.exports.restartLatestVersion = restartNewVersion;

module.exports.installAndRestart = function(restartParams) {
    if(updateInProgress) {
        return Promise.reject("Update in progres");
    }
    updateInProgress = true;
    return squirrel.run(["--update", feedUrl, "--secureUpdates"])
        .then(function() {
            return restartNewVersion(restartParams);
        })
        .catch(function(err) {
            console.warn("Install and restart error", err, err.stack);
            updateInProgress = false;
            return Promise.reject(err);
        });
};
