var fs = require("fs");
var path = require("path");
var util = require("util");

var subprocess = require("../subprocess");
var squirrel = require("./squirrel");

var appName = path.basename(path.resolve(process.execPath, "..", ".."));

var argparseMap = {
  "--squirrel-install": {event: "install", eatArg: true, exitSoon: true},
  "--squirrel-firstrun": {event: "firstrun", eatArg: false, exitSoon: false},
  "--squirrel-uninstall": {event: "uninstall", eatArg: true, exitSoon: true},
  "--squirrel-updated": {event: 'update', eatArg: true, exitSoon: true},
  "--squirrel-obsolete": {event: null, eatArg: true, exitSoon: true},
};


exports.parseStartupInfo = function(argv) {
  var result = {
    updaterEvents: [],
    applicationArgs: [],
    shouldExit: false
  };
  for (var i = 0; i < argv.length; i++) {
		var argInfo = argparseMap[argv[i]];
    if (argInfo) {
      if (argInfo.event) {
        result.updaterEvents.push(argInfo.event);
      }
      if (argInfo.exitSoon) {
        result.shouldExit = true;
      }
			if (argInfo.eatArg) {
        i++;
      }
		} else {
			result.applicationArgs.push(argv[i]);
		}
	}
  return result;
};

var reg_autorun_key = "HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run";
var reg_uninstall_key = "HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall";

module.exports.updateUninstallerIcon = function() {
  var command = util.format(
    'reg add "%s\\%s" /v DisplayIcon /t REG_SZ /d "%s" /f',
    reg_uninstall_key,
    appName,
    process.execPath + ",0"
  );
  return subprocess.shell(command);
};

module.exports.createDefaultShortcuts = function() {
  // C первой попытки может не запуститься. Пробуем несколько раз
  return squirrel.run(["--createShortcut", path.basename(process.execPath)], 5);
};

module.exports.putInAutoRun = function(startArguments) {
  if(!squirrel.path) {
    return Promise.reject("Update.exe not found");
  }
  var start_command = util.format(
    '\\"%s\\" --processStart %s -a \\"%s\\"',
    squirrel.path, path.basename(process.execPath),
    formatArgs(startArguments)
  );
  return subprocess.shell(util.format('reg add "%s" /v "%s" /t REG_SZ /d "%s" /f', reg_autorun_key, appName, start_command));
};

exports.lookupAutoRun = function() {
  return subprocess.shell(util.format('reg query "%s" /v "%s"', reg_autorun_key, appName))
  .then(function(output) {
    var regQueryRe = new RegExp(util.format("\\s+.{%d}\\s+\\S+\\s+(.+)", appName.length));
    var startCommandRe = /"(.+)" --processStart .* -a "(.*)"$/;
    var regMatchResult = output.match(regQueryRe);
    var startCommand = regMatchResult ? regMatchResult[1] : "";
    var startMatchResult = startCommand.match(startCommandRe);
    if (startMatchResult) {
      if (squirrel.path != startMatchResult[1]) {
        return Promise.resolve(null);
      }
      return Promise.resolve({
        startArguments: splitArgs(startMatchResult[2])}
      );
    } else {
      return Promise.resolve(null);
    }
  })
  .catch(function(error) {
    console.warn("Registry autorun lookup error", error);
    return Promise.resolve(null);
  });
};

module.exports.removeFromAutoRun = function() {
  return subprocess.shell(util.format('reg delete "%s" /v "%s" /f', reg_autorun_key, appName));
};

var repeat = function(pattern, times) {
  var result = "";
  for (var i = 0; i < times; i++ ) {
    result += pattern;
  }
  return result;
};

var splitArgs = function(args_line) {
  var quoted = false;
  var escaped = 0;
  var whitespaces = " \t\r\n";
  var flushed = true;
  var currentArg = "";
  var result = [];
  var args_line = args_line.trim();

  var flushEscaped = function() {
    currentArg += repeat("\\", escaped);
    escaped = 0;
  };

  var flushArg = function() {
    flushEscaped();
    result.push(currentArg);
    currentArg = "";
  };

  for (var i = 0; i < args_line.length; i++) {
    var char = args_line[i];
    if (whitespaces.indexOf(char) != -1 && !quoted) {
      if (!flushed) {
        flushArg();
        flushed = true;
      }
      continue;
    }
    flushed = false;
    if (char == "\\") {
      escaped += 1;
    } else if (char != "\"") {
      flushEscaped();
      currentArg += char;
    } else {
      currentArg += repeat("\\", Math.floor(escaped / 2));
      if (escaped % 2) {
        currentArg += "\"";
      } else {
        quoted = !quoted;
      }
      escaped = 0;
    }
  }
  if (!flushed) {
    flushArg();
  }
  return result;
};

var formatArgs = function(args) {
  var escapeQoutes = function(arg) {
    var result = "";
    var backslashes = 0;
    for (var i = 0; i < arg.length; i++) {
      if (arg[i] == "\\") {
        backslashes += 1;
      } else if (arg[i] == "\"") {
        result += repeat("\\", backslashes * 2);
        result += "\\\"";
        backslashes = 0;
      } else {
        if (backslashes) {
          result += repeat("\\", backslashes);
          backslashes = 0;
        }
        result += arg[i];
      }
    }
    return result + repeat("\\", backslashes * 2);
  };

  var result = "";
  for (var i in args) {
    if (args[i].match(/["\s]/) || args[i] == "") {
      result += '"' + escapeQoutes(args[i]) + '" ';
    } else {
      result += args[i] + ' ';
    }
  }
  return result;
};

module.exports.formatArgs = formatArgs;
module.exports.splitArgs = splitArgs;
