'use strict';
var module = angular.module('components.about', []);

var remote = require('remote');
var versionInfo = remote.require('./browser/version-info');

module.controller('AboutController',
['$scope', '$timeout',
function($scope, $timeout) {
  $scope.current = versionInfo.current;
  $scope.updateAvailable = null;
  $scope.error = '';
  $scope.status = remote.process.argv;

  var statusCleanup = null;
  var setStatus = function(value) {
    if (statusCleanup) {
      $timeout.cancel(statusCleanup);
      statusCleanup = null;
    }
    $scope.status = value;
  };

  var clearStatus = function() {
    statusCleanup = $timeout(function() {
      $scope.status = "";
      statusCleanup = null;
    }, 500);
  };

  $scope.checkForUpdates = function() {
    setStatus('checking');
    versionInfo.checkForUpdates()
    .then(function(result) {
      $scope.$apply(function() {
        $scope.updateAvailable = result;
        $scope.error = "";
        clearStatus();
      });
    })
    .catch(function(err) {
      $scope.$apply(function() {
        $scope.error = err;
        clearStatus();
      });
    });
  };

  $scope.installAndRestart = function() {
    setStatus('installing');
    versionInfo.installAndRestart(["--manual-restart"])
    .catch(function(err) {
      $scope.$apply(function() {
        $scope.error = err;
        $scope.updateAvailable = null;
        clearStatus();
      });
    });
  };
}]);
