"use strict";
var module = angular.module('components.device', []);

var remote = require('remote');
var cntserver = remote.require("./browser/cntserver");
var cloud = remote.require("./browser/cloud");
var fileDialog = remote.require("./browser/util/file-dialog");
var log = remote.require("./log");
var util = require("util");

var splitFilePath = function(path) {
    //TODO: Желательно перенести эту логику на сервер
    var normedPath = process.platform === "win32" ? path.replace(/[\\/]/g, "/") : path
    var lastSlashIndex = normedPath.lastIndexOf("/");

    return {
        path: path,
        basePath: lastSlashIndex === -1 ? "" : path.slice(0, lastSlashIndex + 1),
        name: lastSlashIndex === -1 ? path : path.slice(lastSlashIndex + 1)
    };
};


module.controller('DeviceController',
    ['$scope', "$location",
        function($scope, $location) {
            $scope.ready = false;
            $scope.name = null;
            $scope.registered = null;
            $scope.paused = null;
            $scope.excludes = [];
            $scope.includes = [];
            $scope.errors = [];
            $scope.link_device = true;
            $scope.incl_excl = true;
            $scope.show_includes = true;

            var requestPath = $location.url();

            var pushExcludes = function(pathsList) {
                for(var index in pathsList) {
                    $scope.excludes.push(splitFilePath(pathsList[index]));
                }
            };

            const pushIncludes = function(pathsList) {
                for (var index in pathsList) {
                    $scope.includes.push(splitFilePath(pathsList[index]));
                }
            };

            var handleError = function(err) {
                $scope.$apply(function() {
                    // TODO Обработать ошибку
                });
            };

            var getData = function() {

                showPreloader();
                var iterations = 9; // количество попыток
                var interval = 1 * 1000; // 1 sec
                var timer_id = null;

                var try_get_status = function () {
                    cntserver.request("GET", requestPath)
                        .then(function(data) {
                            $scope.$apply(function() {
                                //log.info(util.format("DEVICE: reg = %s", data.registered));
                                if (data.registered !== null || iterations === 0) {
                                    $scope.ready = true;
                                    $scope.name = data.name;
                                    $scope.registered = data.registered;
                                    $scope.type = data.type;
                                    $scope.paused = data.paused;
                                    pushIncludes(data.includes);
                                    pushExcludes(data.excludes);
                                    hidePreloader();
                                    clearTimeout(timer_id);
                                } else {
                                    iterations--;
                                    timer_id = setTimeout(try_get_status, interval);
                                }
                            });
                        })
                        .catch(handleError);
                };

                try_get_status();
            };

            var showPreloader = function() {
                document.getElementById("preloader").style.display = "block";
                document.getElementById("device-body").style.visibility = "hidden";
            };

            var hidePreloader = function() {
                // Проверяем, успел ли сервер перезапуститься. Если да - показываем интерфейс, если нет - пингуем.
                cntserver.request('GET', '/alive')
                    .then(function() {
                        document.getElementById("preloader").style.display = "none";
                        document.getElementById("device-body").style.visibility = "visible";
                        if ($scope.includes.length === 0) {
                            // Значит пользователь удалил все инклюды, и cntserver добавит ему дефолтные.
                            // Нужно показать их пользователю.
                            getData();
                        }
                    })
                    .catch(hidePreloader);
            };

            var syncExcludes = function() {
                showPreloader();
                cntserver.request(
                    "PUT",
                    requestPath + "/excludes",
                    $scope.excludes.map(function(excludeItem) { return excludeItem.path; })
                )
                    .then(cntserver.stop)
                    .then(cntserver.start)
                    .then(hidePreloader)
                    .catch(handleError);
            };

            const syncIncludes = function() {
                showPreloader();
                cntserver.request(
                    "PUT",
                    requestPath + "/includes",
                    $scope.includes.map(function(includeItem) { return includeItem.path; })
                )
                    .then(cntserver.stop)
                    .then(cntserver.start)
                    .then(hidePreloader)
                    .catch(handleError);
            };

            getData();

            $scope.link = function() {
                showPreloader();
                cntserver.request("POST", requestPath + "/link")
                    .then(function() {
                        $scope.$apply(function() {
                            $scope.registered = true;
                            hidePreloader()
                        });
                    })
                    .catch(handleError);
            };

            $scope.unlink = function() {
                cntserver.request("POST", requestPath + "/unlink")
                    .then(function() {
                        $scope.$apply(function() {
                            $scope.registered = false;
                        });
                    })
                    .catch(handleError);
            };

            $scope.addExclude = function() {
                fileDialog.selectFolders()
                    .then(function(selection) {
                        $scope.$apply(function() {
                            pushExcludes(selection);
                        });
                    })
                    .then(syncExcludes)
                    .catch(handleError);
            };

            $scope.addInclude = function() {
                fileDialog.selectFolders()
                    .then(function(selection) {
                        $scope.$apply(function() {
                            pushIncludes(selection);
                        });
                    })
                    .then(syncIncludes)
                    .catch(handleError);
            };

            $scope.removeExclude = function(index) {
                $scope.excludes.splice(index, 1);
                syncExcludes();
            };

            $scope.removeInclude = function(index) {
                $scope.includes.splice(index, 1);
                syncIncludes();
            };

            $scope.toggleTab = function(tab) {
                $scope.show_includes = false;
                $scope.show_excludes = false;
                $scope[tab] = true;
            };

            $scope.showOnline = function() {
                // TODO: поменять на правильную страницу когда появится в продакшене
                cloud.home();
            }

        }
    ]);
