"use strict";
var module = angular.module('components.permit', []);
var remote = require('remote');
var cntserver = remote.require("./browser/cntserver");
var mainWindow = remote.require("./browser/windows/main");
var permitWindow = remote.require("./browser/windows/permit");
var fileDialog = remote.require("./browser/util/file-dialog");
var setPermitConfig = remote.require('./config/config').setPermitConfig;
var getPermitConfig = remote.require('./config/config').getPermitConfig;
var set_permited = remote.require('./browser/findo-app').set_permited;
var show_onboarding = remote.require('./browser/findo-app').show_onboarding;
var notify = remote.require("./browser/tracking-notifier").notify;


var splitFilePath = function(path) {
    //TODO: Желательно перенести эту логику на сервер
    var normedPath = process.platform === "win32" ? path.replace(/[\\/]/g, "/") : path;
    var lastSlashIndex = normedPath.lastIndexOf("/");
    return {
        path: path,
        basePath: lastSlashIndex === -1 ? "" : path.slice(0, lastSlashIndex + 1),
        name: lastSlashIndex === -1 ? path : path.slice(lastSlashIndex + 1)
    }
};

module.controller('PermitController',
    ['$scope', '$location', '$routeParams',
        function($scope, $location, $routeParams) {

            $scope.ready = false;
            $scope.name = null;
            $scope.registered = null;
            $scope.paused = null;
            $scope.excludes = [];
            $scope.includes = [];
            $scope.errors = [];
            $scope.link_device = true;
            $scope.incl_excl = false;
            $scope.show_includes = true;

            var requestPath = "/devices/" + $routeParams['deviceId'];
            var pushExcludes = function(pathsList) {
                for(var index in pathsList) {
                    $scope.excludes.push(splitFilePath(pathsList[index]));
                }
            };

            const pushIncludes = function(pathsList) {
                for (var index in pathsList) {
                    $scope.includes.push(splitFilePath(pathsList[index]));
                }
            };

            var handleError = function(err) {
                $scope.$apply(function() {
                    // TODO Обработать ошибку
                });
            };

            var getData = function() {
                cntserver.request("GET", requestPath)
                    .then(function(data) {
                        $scope.$apply(function() {
                            $scope.ready = true;
                            $scope.name = data.name;
                            $scope.registered = data.registered;
                            $scope.type = data.type;
                            $scope.paused = data.paused;
                            pushIncludes(data.includes);
                            pushExcludes(data.excludes);
                        });
                    })
                    .catch(handleError);
            };

            getData();

            var showPreloader = function() {
                document.getElementById("preloader").style.display = "block";
                document.getElementById("permit-body").style.visibility = "hidden";
            };

            var hidePreloader = function() {
                // Проверяем, успел ли сервер перезапуститься. Если да - показываем интерфейс, если нет - пингуем.
                cntserver.request('GET', '/alive')
                    .then(function() {
                        document.getElementById("preloader").style.display = "none";
                        document.getElementById("permit-body").style.visibility = "visible";
                        if ($scope.includes.length === 0) {
                            // Значит пользователь удалил все инклюды, и cntserver добавит ему дефолтные.
                            // Нужно показать их пользователю.
                            getData();
                        }
                    })
                    .catch(hidePreloader);
            };

            const syncIncludes = function() {
                showPreloader();
                cntserver.request(
                    "PUT",
                    requestPath + "/includes",
                    $scope.includes.map(function(includeItem) { return includeItem.path; })
                )
                    .then(cntserver.stop)
                    .then(cntserver.start)
                    .then(hidePreloader)
                    .catch(handleError);
            };

            var syncExcludes = function() {
                showPreloader();
                cntserver.request(
                    "PUT",
                    requestPath + "/excludes",
                    $scope.excludes.map(function(excludeItem) { return excludeItem.path; })
                )
                    .then(cntserver.stop)
                    .then(cntserver.start)
                    .then(hidePreloader)
                    .catch(handleError);
            };

            $scope.removeInclude = function(index) {
                $scope.includes.splice(index, 1);
                syncIncludes();
            };

            $scope.addInclude = function() {
                fileDialog.selectFolders()
                    .then(function(selection) {
                        $scope.$apply(function() {
                            pushIncludes(selection);
                        });
                    })
                    .then(syncIncludes)
                    .catch(handleError);
            };

            $scope.removeExclude = function(index) {
                $scope.excludes.splice(index, 1);
                syncExcludes();
            };

            $scope.addExclude = function() {
                fileDialog.selectFolders()
                    .then(function(selection) {
                        $scope.$apply(function() {
                            pushExcludes(selection);
                        });
                    })
                    .then(syncExcludes)
                    .catch(handleError);
            };

            $scope.link = function() {
                if ($scope.link_device) {
                    notify("window_connect_next", {"connect_pc":"true"});
                    showPreloader();
                    cntserver.request("POST", requestPath + "/link")
                        .then(function () {
                            $scope.$apply(function () {
                                set_permited();
                                $scope.registered = true;
                                if (getPermitConfig().show_onboarding) {
                                    show_onboarding();
                                    permitWindow.close()
                                } else {
                                    mainWindow.show();
                                    permitWindow.close()
                                }
                            });
                        })
                        .then(hidePreloader)
                        .catch(handleError);
                } else {
                    notify("window_connect_next", {"connect_pc":"false"});
                    if (getPermitConfig().show_onboarding) {
                        show_onboarding();
                        permitWindow.close()
                    } else {
                        mainWindow.show();
                        permitWindow.close()
                    }
                }
            };

            $scope.show_incl_excl = function() {
                $scope.incl_excl = !$scope.incl_excl;
            };

            $scope.toggleTab = function(tab) {
                $scope.show_includes = false;
                $scope.show_excludes = false;
                $scope[tab] = true;
            }

        }]);
